@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-from typing import Awaitable
+import asyncio
 from selve.commandClasses.command import CommeoCommandDevice, CommeoCommandGroup, CommeoCommandGroupMan
 from selve.commandClasses.common import CommeoParamGetEvent, CommeoParamSetEvent, CommeoServiceFactoryReset, CommeoServiceGetState, CommeoServiceGetVersion, CommeoServicePing, CommeoServiceReset
 from selve.commandClasses.sensor import CommeoSensorGetIDs, SensorDevice
@@ -47,7 +47,10 @@ class Gateway():
             exit()
 
         if discover:
-            self.discover()
+            _LOGGER.info("Discovering devices")
+            loop = asyncio.new_event_loop()
+            asyncio.set_event_loop(loop)
+            result = loop.run_until_complete(self.discover())
 
         #self.readThread = threading.Thread(target=self.readFromPort)
         #self.readThread.start()
@@ -123,16 +126,6 @@ class Gateway():
         
 
     async def executeCommand(self, command):
-        a = CommeoServiceGetState()
-        await self._executeCommand(a)
-        _LOGGER.debug("Service State: " + str(a.status))
-        if a.status == ServiceState.READY:
-            return self._executeCommand(command)
-        else:
-            _LOGGER.error("Gateway not ready to execute command")
-        return None
-
-    async def _executeCommand(self, command):
         """[summary]
         Execute the given command using the serial port.
         It opens a communication to the serial port each time a
@@ -182,6 +175,9 @@ class Gateway():
         """[summary]
             Discover all devices registered on the usb-commeo        
         """
+        await self.gatewayReady()
+            
+
         commandIveo = IveoCommandGetIds()
         commandCommeoActors = CommeoDeviceGetIDs()
         commandCommeoGroups = CommeoGroupGetIDs()
@@ -233,42 +229,42 @@ class Gateway():
             iveoDevices = {}
         else:
             _LOGGER.debug(f'discover ids: {commandIveo.ids}')
-            iveoDevices = dict([(id, IveoDevice(self, id, True) )for id in commandIveo.ids])
+            iveoDevices = dict([(id, IveoDevice(self, id) )for id in commandIveo.ids])
         
         if not hasattr(commandCommeoActors, "ids"):
             _LOGGER.info("Associated Commeo Devices not found") 
             commeoActors = {}
         else:
             _LOGGER.debug(f'discover ids: {commandCommeoActors.ids}')
-            commeoActors = dict([(id, ActorDevice(self, id, True) )for id in commandCommeoActors.ids])
+            commeoActors = dict([(id, ActorDevice(self, id) )for id in commandCommeoActors.ids])
         
         if not hasattr(commandCommeoGroups, "ids"):
             _LOGGER.info("Associated Commeo Groups not found") 
             commeoGroups = {}
         else:
             _LOGGER.debug(f'discover ids: {commandCommeoGroups.ids}')
-            commeoGroups = dict([(id, GroupDevice(self, id, True) )for id in commandCommeoGroups.ids])
+            commeoGroups = dict([(id, GroupDevice(self, id) )for id in commandCommeoGroups.ids])
         
         if not hasattr(commandCommeoSenders, "ids"):
             _LOGGER.info("Associated Commeo Devices not found") 
             commeoSenders = {}
         else:
             _LOGGER.debug(f'discover ids: {commandCommeoSenders.ids}')
-            commeoSenders = dict([(id, SenderDevice(self, id, True) )for id in commandCommeoSenders.ids])
+            commeoSenders = dict([(id, SenderDevice(self, id) )for id in commandCommeoSenders.ids])
         
         if not hasattr(commandCommeoSenSims, "ids"):
             _LOGGER.info("Associated Commeo Devices not found") 
             commeoSenSims = {}
         else:
             _LOGGER.debug(f'discover ids: {commandCommeoSenSims.ids}')
-            commeoSenSims = dict([(id, SenSimDevice(self, id ,True) )for id in commandCommeoSenSims.ids])
+            commeoSenSims = dict([(id, SenSimDevice(self, id) )for id in commandCommeoSenSims.ids])
         
         if not hasattr(commandCommeoSensors, "ids"):
             _LOGGER.info("Associated Commeo Devices not found") 
             commeoSensors = {}
         else:
             _LOGGER.debug(f'discover ids: {commandCommeoSensors.ids}')
-            commeoSensors = dict([(id, SensorDevice(self, id , True) )for id in commandCommeoSensors.ids])
+            commeoSensors = dict([(id, SensorDevice(self, id) )for id in commandCommeoSensors.ids])
         
 
         self.devices.update(iveoDevices) 
@@ -278,6 +274,9 @@ class Gateway():
         self.devices.update(commeoSenSims) 
         self.devices.update(commeoSensors)
         
+        for id, device in self.devices.items():
+            await device.discover_properties()
+        
         self.list_devices() 
        
     def addDevice(self, id, device):
@@ -316,8 +315,9 @@ class Gateway():
            
     ## Common ##
     async def pingGateway(self):
+        self.gatewayReady()
         command = CommeoServicePing()
-        command.execute(self)
+        await command.execute(self)
         print("Ping")
 
     # GATEWAY STATE....
@@ -328,7 +328,14 @@ class Gateway():
         if hasattr(command, "status"):
             return command.status.name
 
+    async def gatewayReady(self):
+        if await self.gatewayState() == "READY":
+            return
+        else:
+            raise GatewayError
+
     async def getVersionG(self):
+        self.gatewayReady()
         command = CommeoServiceGetVersion()
         await command.execute(self)
         return command
@@ -343,6 +350,7 @@ class Gateway():
         return self.getVersionG.spec
 
     async def resetGateway(self):
+        self.gatewayReady()
         command = CommeoServiceReset()
         await command.execute(self)
         while not command.executed:
@@ -362,6 +370,7 @@ class Gateway():
             _LOGGER.info("Gateway reset")
 
     async def factoryResetGateway(self):
+        self.gatewayReady()
         command = CommeoServiceFactoryReset()
         await command.execute(self)
         if command.executed:
@@ -370,10 +379,12 @@ class Gateway():
             _LOGGER.info("Factory reset failed")
 
     async def setEvents(self, eventDevice, eventSensor, eventSender, eventLogging, eventDuty):
+        self.gatewayReady()
         command = CommeoParamSetEvent(eventDevice, eventSensor, eventSender, eventLogging, eventDuty)
         command.execute(self)
 
     async def getEvents(self):
+        self.gatewayReady()
         command = CommeoParamGetEvent()
         await command.execute(self)
         _LOGGER.debug("Events: " + str(command.eventDevice) + " " + str(command.eventSensor) + " " + str(command.eventSender) + " " + str(command.eventLogging) + " " + str(command.eventDuty))
@@ -381,11 +392,13 @@ class Gateway():
 ## Actor
 
     async def sendCommandToActor(self, id, command, type=DeviceCommandTypes.MANUAL, parameter=0):
+        self.gatewayReady()
         command = CommeoCommandDevice(id, command, type, parameter)
         await command.execute(self)
         return command.executed
 
     async def scanActorDevices(self):
+        self.gatewayReady()
         commandStart = CommeoDeviceScanStart()
         commandStop = CommeoDeviceScanStop()
         commandResult = CommeoDeviceScanResult()
@@ -402,10 +415,11 @@ class Gateway():
         return {}
 
     async def saveActorDevices(self, ids):
+        self.gatewayReady()
         if len(ids) > 0:
             for id in ids:
                 commandSave = CommeoDeviceSave(id)
-                commandSave.execute(self)
+                await commandSave.execute(self)
                 dev = ActorDevice(self, id , True)
                 self.addDevice(id, dev)
                 dev.saveDevice()
@@ -418,11 +432,17 @@ class Gateway():
 ## Group
 
     async def sendCommandToGroup(self, id, command, type=DeviceCommandTypes.MANUAL, parameter=0):
+        self.gatewayReady()
         command = CommeoCommandGroup(id, command, type, parameter)
         await command.execute(self)
         return command.executed
 
     async def sendCommandToGroupMan(self, idMask, command, type=DeviceCommandTypes.MANUAL, parameter=0):
+        self.gatewayReady()
         command = CommeoCommandGroupMan(command, type, idMask, parameter)
         await command.execute(self)
-        return command.ids
\ No newline at end of file
+        return command.ids
+
+
+class GatewayError(Exception):
+    pass
